function y = b_spline(t,p)
%                   1-D B-SPLINE
% =============================================================================
% CALLING: y = b_spline(t,p)
% 1) y     = b_spline(t,p) ... evaluation
% 2) yINFO = b_spline      ... getting information structure
% -----------------------------------------------------------------------------
% t   ... (N x m) matrix of N points at which G(t) is to be evaluated:
%         t(n,:)=[t(n,1),...,t(n,m)]...coordinates of n-th point in
%         m-dimensional space, m=1.
% p   ... (Np x k) matrix of Np parameter sets for which G(t) is to be
%         evaluated, k<=K:=4 so far:
%         a:=p(j,1) ... j-th shift            (optional: a=0 by default)
%         b:=p(j,2) ... j-th width            (optional: b=1 by default)
%         c:=p(j,3) ... j-th modulation freq. (optional: c=0 by default)
%         d:=p(j,4) ... order                 (optional: d=2 by default)
%         a,b,c,d are scalars.
% y   ... N x Np matrix of 'b_spline' evaluations at N points and Np parameter sets:
%         y(n,j) =  b_spline(t(n,:),p(j,:)) ...
%                  ... value at n-th point for j-th parameter set
% yINFO ... information structure about 'b_spline': see help fun for more details
%

% (C) 2010, J. Spirik, University of Technology Brno (Czech Republic)
%
error(nargchk(0,2,nargin));
if nargin<2, p=[]; end
yINFO = struct(...
     'type',          'kernel',...   % 'kernel', 'mother/father wavelet', 'heaviside', unknown'
     'dim',           1,...          %  dimension
     'name',          'B-spline',... % verbose function name
     'p_names',       {{'shift','width','modulation freq','order'}},...% parameter names
     'p_defaults',    [0,1,0,2],...    % parameter defaults
     'y'   ,          'b_spline',...    % this function
     'yDUAL',         '',...         % dual father/mother wavelet (only with type wavelet)
     'yFT' ,          'b_splineFT' ,... % its Fourier transform
     'yACF',          'b_splineACF',... % its autocorrelation function
     'symmetry',      'even',...     % 'even', 'odd', 'none'
     'support' ,      [-Inf,Inf],... % support(j,:)=support intervals in j-th dim.
     'sing_val_at',   [],...         % positions of singular values (# x m)
     'extreme_val',   1,...          % global extreme
     'extreme_val_at',0,...          % positions of global extreme (# x m)
     'plotaxis',      [-1,1,0,1],... % plot axis scaling
     'FWHM',          1);           % full width at half maximum
 
if nargin<1, y = yINFO; return; end    % Getting yINFO structure:
%
% CHECKS:
szt = size(t);
szp = size(p);
if length(szt)>2 | length(szp)>2
   error('One of ''t'' or ''p'' is not a matrix');
end
if szt(1)==1, t=reshape(t,szt(2),1); szt=size(t); end % for m==1 only
N = szt(1);
m = szt(2);
if m~=yINFO.dim
   error(['Number of columns in ''t'' differs from dimension ',...
          num2str(yINFO.dim)]);
end
K = length(yINFO.p_defaults);
Np= max(1,szp(1));
k = szp(2);
if k>K
   error(['Number of columns (parameters) in ''p'' must not exceed ',...
          num2str(K)]);
end

if min(p(:,4))<2
    error('Order of B-spline must be minimum 2.');
end
%
% SETTING DEFAULTS:
  p = [p,kron(ones(Np,1),yINFO.p_defaults(k+1:K))];
%
% EVALUATION:
  tp = meshprod({1:N,1:Np});
  ti = tp(:,1); pj = tp(:,2); clear tp;
  
  
y = length(ti);
for i=1:length(ti)
    y(i)=0;
    t(ti(i))=t(ti(i))/p(pj(i),2);
    for j=0:p(pj(i),4)
        if ((t(ti(i))+(p(pj(i),4)/2)-j-p(pj(i),1))>0) 
        y(i)=y(i)+(((-1).^j)*(factorial(p(pj(i),4))/(factorial(j)*factorial(p(pj(i),4)-j)))*((t(ti(i))+(p(pj(i),4)/2)-j-p(pj(i),1)).^(p(pj(i),4)-1))); 
        end
    end
    y(i)=(1/factorial(p(pj(i),4)-1))*y(i);
    if y(i)<10^(-13)
        y(i)=0;
    end
end
  y = cos(2*pi*p(pj,3).*t(ti)).*y';
  y = reshape(y,N,Np);


