function y = bezier(t,p,P)
%                   BEZIER CURVE
% =============================================================================
% CALLING: y = bezier(t,p,P)
% 1) y     = bezier(t,p,P) ... evaluation
% 2) yINFO = bezier      ... getting information structure
% -----------------------------------------------------------------------------
% t   ... (N x m) matrix of N points at which G(t) is to be evaluated:
%         t(n,:)=[t(n,1),...,t(n,m)]...coordinates of n-th point in
%         m-dimensional space, m=1.
% p   ... (Np x k) matrix of Np parameter sets for which G(t) is to be
%         evaluated, k<=K:=3 so far:
%         a:=p(j,1) ... j-th shift            (optional: a=0 by default)
%         b:=p(j,2) ... j-th width            (optional: b=1 by default)
%         c:=p(j,3) ... j-th modulation freq. (optional: c=0 by default)
%         a,b,c are scalars.
% P   ... (K x (m+1)) matrix of K control points
% yINFO ... information structure about 'bezier': see help fun for more details
%

% (C) 2010, J. Spirik, University of Technology Brno (Czech Republic)
%
error(nargchk(0,3,nargin));
if nargin==1, p=[0,1,0]; P=[0,0;1,1]; end
if nargin<3, P=[0,0;1,1]; end
yINFO = struct(...
     'type',          'kernel',...   % 'kernel', 'mother/father wavelet', 'heaviside', unknown'
     'dim',           1,...          %  dimension
     'name',          'Bezier curve',... % verbose function name
     'p_names',       {{'shift','width','modulation freq'}},...% parameter names
     'p_defaults',    [0,1,0],...    % parameter defaults
     'y'   ,          'bezier',...    % this function
     'yDUAL',         '',...         % dual father/mother wavelet (only with type wavelet)
     'yFT' ,          'bezierFT' ,... % its Fourier transform
     'yACF',          'bezierACF',... % its autocorrelation function
     'symmetry',      'none',...     % 'even', 'odd', 'none'
     'support' ,      [-Inf,Inf],... % support(j,:)=support intervals in j-th dim.
     'sing_val_at',   [],...         % positions of singular values (# x m)
     'plotaxis',      [-1,1,0,1]); % plot axis scaling
     
if nargin<1, y = yINFO; return; end    % Getting yINFO structure:


szt = size(t);
szp = size(p);
if length(szt)>2 | length(szp)>2
   error('One of ''t'' or ''p'' is not a matrix');
end
K = length(yINFO.p_defaults);
Np= max(1,szp(1));
k = szp(1);
if k>K
   error(['Number of columns (parameters) in ''p'' must not exceed ',...
          num2str(K)]);
end

G1=[-1,1;1,0];
G2=[1,-2,1;-2,2,0;1,0,0];
G3=[-1,3,-3,1;3,-6,3,0;-3,3,0,0;1,0,0,0];
G4=[1,-4,6,-4,1;-4,12,-12,4,0;6,-12,6,0,0;-4,4,0,0,0;1,0,0,0,0];
G5=[-1,5,-10,10,-5,1;5,-20,30,-20,5,0;-10,30,-30,10,0,0;10,-20,10,0,0,0;-5,5,0,0,0,0;1,0,0,0,0,0];

P=sortrows(P);
N=length(P)-1;
y=zeros(1,length(t));
 
switch N
    case {1}
        G=G1;
    case {2}
        G=G2;
    case {3}
        G=G3;
    case {4}
        G=G4;
    case {5}
        G=G5;
    otherwise
        error('Order must be between 1 and 5.');
end
Pold=P;
center=(P(1,1)+P(end,1))/2;

for a=1:Np
 P(:,1)=(Pold(:,1)).*p(a,2);
 center2=(P(1,1)+P(end,1))/2;
 P(:,1)=P(:,1)-(center2-center);
 P(:,1)=P(:,1)++p(a,1);
 koef=G*P(:,1);   
for k=1:length(t)
    if(t(k)<P(1,1) || t(k)>P(end,1))
        y(a,k)=0;
    else
    koefx=koef;
    koefx(end)=koefx(end)-(t(k));
    rx = roots(koefx);
    for i=1:length(rx)
        if rx(i)<=1 && rx(i)>=0
            T=rx(i);
        end
    end
    tmat=zeros(1,N+1);
    for i=0:N
        tmat(i+1)=T^(N-i);
    end
    y(a,k)=tmat*G*P(:,2);
    end
end
  y(a,:) = cos(2*pi*p(a,3)*t).*y(a,:);
end
y=y';
