function y = daubechies(t,p)
%                   1-D Daubechies wavelet
% =============================================================================
% CALLING: y = daubechies(t,p)
% 1) y     = daubechies(t,p) ... evaluation
% 2) yINFO = daubechies      ... getting information structure
% -----------------------------------------------------------------------------
% t   ... (N x m) matrix of N points at which G(t) is to be evaluated:
%         t(n,:)=[t(n,1),...,t(n,m)]...coordinates of n-th point in
%         m-dimensional space, m=1.
% p   ... (Np x k) matrix of Np parameter sets for which G(t) is to be
%         evaluated, k<=K:=4 so far:
%         a:=p(j,1) ... j-th shift                     (optional: a=0 by default)
%         b:=p(j,2) ... j-th width                     (optional: b=1 by default)
%         c:=p(j,3) ... j-th order                     (optional: c=2 by default)
%         d:=p(j,4) ... j-th wavelet/scaling function  (optional: d=0 by default)
%         a,b,c,d are scalars.
% y   ... N x Np matrix of 'daubechies' evaluations at N points and Np parameter sets:
%         y(n,j) = daubechies(t(n,:),p(j,:)) ...
%                  ... value at n-th point for j-th parameter set
% yINFO ... information structure about 'daubechies': see help fun for more details
%

% (C) 2010, J. Spirik, University of Technology Brno (Czech Republic)
%
error(nargchk(0,2,nargin));
if nargin<2, p=[]; end
yINFO = struct(...
     'type',          'mother wavelet',...   % 'kernel', 'mother/father wavelet', 'heaviside', unknown'
     'dim',           1,...          %  dimension
     'name',          'Daubechies wavelet',... % verbose function name
     'p_names',       {{'shift','width','order','wavelet/scaling function'}},...% parameter names
     'p_defaults',    [0,1,2,0],...    % parameter defaults
     'y'   ,          'Daubechies wavelet',...    % this function
     'yDUAL',         '',...         % dual father/mother wavelet (only with type wavelet)
     'yACF',          'bsplineACF',... % its autocorrelation function
     'symmetry',      'none',...     % 'even', 'odd', 'none'
     'support' ,      [-Inf,Inf],... % support(j,:)=support intervals in j-th dim.
     'plotaxis',      [0,3,-2,2]); % plot axis scaling

if nargin<1, y = yINFO; return; end    % Getting yINFO structure:
%
% CHECKS:
szt = size(t);
szp = size(p);
if length(szt)>2 | length(szp)>2
   error('One of ''t'' or ''p'' is not a matrix');
end
if szt(1)==1, t=reshape(t,szt(2),1); szt=size(t); end % for m==1 only
N = szt(1);
m = szt(2);
if m~=yINFO.dim
   error(['Number of columns in ''t'' differs from dimension ',...
          num2str(yINFO.dim)]);
end
K = length(yINFO.p_defaults);
Np= max(1,szp(1));
k = szp(2);
if k>K
   error(['Number of columns (parameters) in ''p'' must not exceed ',...
          num2str(K)]);
end

% SETTING DEFAULTS:
  p = [p,kron(ones(Np,1),yINFO.p_defaults(k+1:K))];

for a=1:Np
    N=p(a,3);
    if p(a,4)==0
        c = 0;
        d = 1; 
        else
        c = 1;
        d = 0; 
    end
for k=[0:N-1]
P(k+1) = factorial(N + k -1)/(factorial(k)*factorial(N-1));
end
% P
Q = zeros(1,2*N-1);
Cnvl = [1]; 
for i=[1:N]
    Shift = zeros(1,N - i + 1);
    Shift(1) = 1;
    Add = conv((P(i)/4^(i-1))*Cnvl,Shift);
        for j=[1:size(Add,2)]
        Q(j) = Q(j) + Add(size(Add,2)-j + 1);
        end
    Cnvl = conv(Cnvl,[-1 2 -1]);
end
% Q
Qp = roots(Q);
for i=[1:size(Qp,1)]
    if Qp(i)*conj(Qp(i))<=1
    Qp(i)=0;
    end
end
Q1 = poly(Qp(find(Qp)));
m0 = Q1/sum(Q1);
for i=[1:N]
    m0 = conv(m0,[1/2 1/2]);
end
m0;
for i=[1:size(m0,2)]
    C(i) = m0(size(m0,2)-i+1)*sqrt(2);
end


dfilt = sqrt(2)*C;
for i=[1:size(dfilt,2)]
dfilt2(i) = dfilt(size(dfilt,2)-i+1)*(-1)^(i+1);
end
c = conv(c,dfilt) + conv(d,dfilt2); 
for i=1:9
len = length(c);
mul = [];
mul(1:len,1:2:2*len-1)=eye(len); 
c = conv(c*mul,dfilt);
end
c(2,:)= (0:(2*N-1)/(length(c)-1):2*N-1);

y(:,a)=interp1(c(2,:),c(1,:),(t'-p(a,1))/p(a,2),'linear',0);

end



