function y = rectfun(t,p)
%                   1-D RECTANGLE PULSE FUNCTION
% =============================================================================
% CALLING: y = rectfun(t,p)
% 1) y     = rectfun(t,p) ... evaluation
% 2) yINFO = rectfun      ... getting information structure
% -----------------------------------------------------------------------------
% t   ... (N x m) matrix of N points at which rectfun(t) is to be evaluated:
%         t(n,:)=[t(n,1),...,t(n,m)]...coordinates of n-th point in
%         m-dimensional space, m=1.
% p   ... (Np x k) matrix of Np parameter sets for which L(t) is to be
%         evaluated, k<=K:=2 so far:
%         a:=p(j,1) ... j-th shift            (optional: a=0 by default)
%         b:=p(j,2) ... j-th width            (optional: b=1 by default)
%         a,b are scalars.
%         rectfun(t(n,:),p(j,:)) = a*L((t(n,:)-a)./b)
% y   ... N x Np matrix of 'rectfun' evaluations at N points and Np parameter sets:
%         y(n,j) = rectfun(t(n,:),p(j,:)) ...
%                  ... value at n-th point for j-th parameter set
% yINFO ... information structure about 'loren': see help fun for more details
%

% (C) 2009, J. Spirik, University of Technology Brno (Czech Republic)
%
error(nargchk(0,2,nargin));
if nargin<2, p=[]; end
yINFO = struct(...
     'type',          'kernel',...   % 'kernel', 'mother/father wavelet', 'heaviside', unknown'
     'dim',           1,...          %  dimension
     'name',          'Rectangle Pulse function',... % verbose function name
     'p_names',       {{'shift','width'}},...% parameter names
     'p_defaults',    [0,1],...      % parameter defaults
     'y'   ,          'rectfun',...    % this function
     'yDUAL',         '',...         % dual father/mother wavelet (only with type wavelet)
     'yFT' ,          'rectfunFT' ,... % its Fourier transform
     'yACF',          'rectfunACF',... % its autocorrelation function
     'symmetry',      'even',...     % 'even', 'odd', 'none'
     'support' ,      [-Inf,Inf],... % support(j,:)=support intervals in j-th dim.
     'sing_val_at',   [],...         % positions of singular values (# x m)
     'extreme_val',   1,...          % global extreme
     'extreme_val_at',0,...          % positions of global extreme (# x m)
     'plotaxis',      [-2,2,0,1] ,... % plot axis scaling 
     'FWHM',          1,...          % full width at half maximum
     'L1norm',        1,...             % L^1 norm 
     'L2norm',        1,...           % L^2 norm
     'TV',            2);            % total variation
if nargin<1, y = yINFO; return; end    % Getting yINFO structure:
%
% CHECKS:
szt = size(t);
szp = size(p);
if length(szt)>2 | length(szp)>2
   error('One of ''t'' or ''p'' is not a matrix');
end
if szt(1)==1, t=reshape(t,szt(2),1); szt=size(t); end % for m==1 only
N = szt(1);
m = szt(2);
if m~=yINFO.dim
   error(['Number of columns in ''t'' differs from dimension ',...
          num2str(yINFO.dim)]);
end
K = length(yINFO.p_defaults);
Np= max(1,szp(1));
k = szp(2);
if k>K
   error(['Number of columns (parameters) in ''p'' must not exceed ',...
          num2str(K)]);
end
%
% SETTING DEFAULTS:
  p = [p,kron(ones(Np,1),yINFO.p_defaults(k+1:K))];
%
% EVALUATION:
  tp = meshprod({1:N,1:Np});
  ti = tp(:,1); pj = tp(:,2); clear tp;
% Basic 1-D shape shifted and scaled:
  y = rectpuls((t(ti)-p(pj,1)),p(pj,2));


  y = reshape(y,N,Np);
